# Documentation string. I wonder what ever refers to it? 

__doc__ = """NFGnav product module.

This module provides the methods and classes that Zope needs for the product. 
__init__.py is responsible for registering these methods and classes.

See README.txt for more details."""

# Version string. Usually updated automatically by CVS. 

__version__ = '0.7.1' # I used to wonder whether Zope checked these. Nope. 

# Now, you can compress these imports down to three lines, but that makes it 
# harder to document it as you go. Here's the long-winded version: 

from Globals import HTMLFile	  # fakes a method from a DTML file
from Globals import MessageDialog # provid
from Globals import Persistent	# makes an object stick in the ZODB

import OFS.SimpleItem
import Acquisition
import AccessControl.Role
import string
import re

"""Step #1: Provide "constructors" for Zope's management interface. 
__init__ is all very well and good for Python, but doesn't provide 
a nice HTML interface or accept arguments from an HTTP request, so 
you need to provide some functions to act as a middle-man between 
Zope and your class.

Use HTMLFile to produce a [pseudo-]function which hands back an 
appropriate DTML document based on a file sitting in your product 
directory. 
"""

manage_addNFGnavForm = HTMLFile('NFGnav_Add', globals()) 
manage_addTreeMenuForm = HTMLFile('TreeMenu_Add', globals())

def manage_addNFGnav(self, id, title='', REQUEST=None):
	"""Add a NFGnav to a folder."""
	self._setObject(id, NFGnav(id, title))
	if REQUEST is not None:
		return self.manage_main(self, REQUEST)

def manage_addTreeMenu(self, id, title='', REQUEST=None):
	"""Create a TreeMenu instance."""
	self._setObject(id, TreeMenu(id, title))
	if REQUEST is not None:
		return self.manage_main(self, REQUEST)
	
class NFGnav(
	OFS.SimpleItem.Item, 
	Persistent,
	Acquisition.Implicit,
	AccessControl.Role.RoleManager
	): 
	"""NFGnav object. 
	"""
		
	meta_type = 'NFGnav' # what do people think they're adding? 

	manage_options = ( # what management options are there?
	{'label': 'Edit',	   'action': 'manage_main'},
	{'label': 'View SiteMap',	   'action': 'view_map'},
	{'label': 'View TreeMenu',	   'action': 'view_menu'}, 
	{'label': 'Security',   'action': 'manage_access'},
		) 

	#	__ac_permissions__=( # what permissions make sense for us? 
	#	('View management screens', ('manage_tabs','manage_main')),
	#	('Change permissions', ('manage_access',) ),
	#	('Change NFGnav', ('manage_edit',),('Manager',) ),
	#	('View NFGnav',('') ),
	#	('Reference in NFGnav',('',),('Manager',) ),
	#	)
	__ac_permissions__=( 
        ('View management screens', ('manage_tabs','manage_main')),
        ('Change permissions',    ('manage_access',)               ),
        ('Change NFG_SiteMaps'   ,       ('manage_edit',)),
	 ('Reference in NFGnav', ('',)),
        ('View',                   ('',)),
        )
	
	def __init__(self, id, title='', REQUEST=None): 
		"""initialise a new instance of NFGnav"""
		self.id = id
		self.title = title

	view_map = HTMLFile('NFGnav_ViewMap', globals())
	view_menu = HTMLFile('NFGnav_ViewMenu', globals())
	
	manage_main = HTMLFile('NFGnav_Edit', globals()) # Management Interface
	
	def manage_edit(self, title, REQUEST=None): 
		"""does this really need a doc string?"""
		self.title = title
		if REQUEST is not None:
			return MessageDialog(
				title = 'Edited',
				message = "Properties for %s changed." % self.id,
				action = './manage_main',
				)

	def TreeMenu(self,
				 PARENTS,
				 REQUEST,
				 callerid=None,
				 context=None,
				 initlevel=1,
				 BaseURL="",
				 Meta_Types=['Folder'],
				 sort='id',
				 maxlevel=None):
		"""A vertical navigation showing all submenus leading up to the current context
		plus the first submenu layer beneath the current context.
		You can filter what types of objects are shown by setting the parameter Meta_Types (default: ['Folder']).
		You can filter which instances are shown by the instance's Security permission 'Reference in NFGnav'.
		Objects are sorted on id. You can specify a different variable name on which objects should be sorted.
		Any object that has your specified variable set will be indexed to that variable, others will be indexed on id.
		You can customize the TreeMenu by implementing some special DTML Methods, see below.
		"""
		return self.ChartLevel(
			PARENTS,
			REQUEST,
			callerid,
			context,
			initlevel,
			BaseURL,
			mode="tree",
			Meta_Types=Meta_Types,
			sort=sort,
			maxlevel=maxlevel)

	def SiteMap(self,
				PARENTS,
				REQUEST,
				callerid=None,
				context=None,
				initlevel=1,
				BaseURL="",
				Meta_Types=['Folder'],
				sort='id',
				maxlevel=None):		
		"""A site map showing all submenu trees expanded.
		You can filter what types of objects are shown by setting the parameter Meta_Types (default: ['Folder']).
		You can filter which instances are shown by the instance's Security permission 'Reference in NFGnav'.
		Objects are sorted on id. You can specify a different variable name on which objects should be sorted.
		Any object that has your specified variable set will be indexed to that variable, others will be indexed on id.
		You can customize the SiteMap by implementing some special DTML Methods, see below.
		"""
		return self.ChartLevel(
			PARENTS,
			REQUEST,
			callerid,
			context,
			initlevel,
			BaseURL,
			mode="map",
			Meta_Types=Meta_Types,
			sort=sort,
			maxlevel=maxlevel)

	def LevelTrail(self,
				   PARENTS,
				   REQUEST,
				   callerid=None,
				   context=None,
				   initlevel=1,
				   BaseURL="",
				   Meta_Types=['Folder'],
				   sort='id',
				   maxlevel=None):		
		"""A very compact menu, showing all (grand)parents of the current item.
		You can filter what types of objects are shown by setting the parameter Meta_Types (default: ['Folder']).
		You can filter which instances are shown by the instance's Security permission 'Reference in NFGnav'.
		You can customize the LevelTrail by implementing some special DTML Methods, see below."""
		return self.ChartLevel(
			PARENTS,
			REQUEST,
			callerid,
			context,
			initlevel,
			BaseURL,
			mode="trail",
			Meta_Types=Meta_Types,
			sort=sort,
			maxlevel=maxlevel)
		
	def ChartLevel(self,
				   PARENTS,
				   REQUEST,
				   callerid=None,
				   context=None,
				   initlevel=1,
				   BaseURL="",
				   mode="map",
				   Meta_Types=['Folder'],
				   sort='id',
				   maxlevel=None):		
		"""The main workhorse method for generating both TreeMenu and SiteMap renderings."""
		level=initlevel
		reverse = initlevel * -1
		nextreverse = reverse -1
		nextlevel = initlevel +1
		if context == None:
			# it is possible to set inconsistent context/initlevel pairs!
			context=PARENTS[reverse]
		retval = ""
		children=[]
		splitsort=string.split(sort,' ')
		sortvar=splitsort[0]
		for (childid, child) in context.objectItems(Meta_Types):
			if sortvar == 'id':
				sortkey = childid
			else:
				try:
					sortkey = getattr(child, sortvar)
				except AttributeError:
					sortkey = childid
			children.append((sortkey, child, childid))
		children.sort()
		if splitsort[-1] == 'reverse':
			children.reverse()
		retval = retval + "%s\n" % self.LevelOpen(PARENTS, REQUEST, level, mode)
		for (sortkey, child, childid) in children:
			if REQUEST.has_key('AUTHENTICATED_USER') \
			   and REQUEST['AUTHENTICATED_USER'].has_permission("Reference in NFGnav",child):
				try:
					# maybe there's a sublevel beneath this?
					newcontext = PARENTS[nextreverse]
					newcontextid = self.extractid(newcontext)
				except IndexError:
					# we're on the deepest level already, back off
					newcontext = None
					newcontextid = None
				if childid == callerid or childid == newcontextid or mode != "trail":
					# show context, unless we're in trail mode and this branches off our trail
					retval = retval + "%s\n" % (self.RenderItem(
						PARENTS,
						REQUEST,
						callerid,
						child,
						newcontext,
						level,
						mode,
						BaseURL))
				if (maxlevel is None or nextlevel <= maxlevel) \
				   and (mode=="map" or childid == newcontextid or childid == callerid):
					retval = retval + "%s" % self.ChartLevel(
						PARENTS,
						REQUEST,
						callerid,
						context=child,
						initlevel=nextlevel,
						BaseURL="%s/%s" % (BaseURL, child.tpURL()),
						mode=mode,
						Meta_Types=Meta_Types,
						sort=sort,
						maxlevel=maxlevel
						)
			else:
				pass
		retval = retval + "%s" % self.LevelClose(PARENTS, REQUEST, level, mode)
		return retval

	def RenderItem(self, PARENTS, REQUEST, callerid, object, newcontext, level, mode="map", BaseURL=""):
		"""
		Scriptable hook for rendering object references in a TreeMenu or SiteMap.
		You can provide DTML methods for customized implementation:
		for SiteMap: 
		>> NFGnav_ActiveMapItem will be used to render an object in the acquisition path.
		>> NFGnav_LinkMapItem will be used to render an object outside the acquisition path.
		for TreeMenu:
		>> NFGnav_ActiveTreeItem will be used to render an object in the acquisition path.
		>> NFGnav_LinkTreeItem will be used to render an object outside the acquisition path.
		In all cases, the following variables will be set:
		NFGnav_Level
		NFGnav_ItemTitle
		NFGnav_ItemURL
		If DTML methods are not found, a default rendering is returned.
		"""
		REQUEST.set('NFGnav_callerid', callerid)		
		REQUEST.set('NFGnav_Level', level)
		REQUEST.set('NFGnav_Item', object)
		# this one is necessary because Folders and DTML Documents
		# have different differences between id, "id" and "id()" ....
		REQUEST.set('NFGnav_ItemID', self.extractid(object)) 
		## not necessary anymore, since we set the object itself:
		REQUEST.set('NFGnav_ItemTitle', object.title_or_id())	# "NFGnav_Item.title_or_id()"
		if object.meta_type == 'Folder':
			itemURL = "%s/%s/" % (BaseURL, object.tpURL())
		else:
			itemURL = "%s/%s" % (BaseURL, object.tpURL())
		REQUEST.set('NFGnav_ItemURL', itemURL)			# "NFGnav_Item.absolute_url()"
		REQUEST.set('NFGnav_ItemIcon', object.icon)			# "NFGnav_Item.icon"

		# This if/elif block with hardcoded DTML method names is not very elegant;
		# it would be nicer to condense Active/Link and Map/Tree into dynamic method names.
		# BUT we'd have to do eval() to arrive at the actual method names
		# and I don't want to create huge security holes. So let's keep it dumb:
		if newcontext is not None:
				newcontextid = self.extractid(newcontext)
		else:
			newcontextid = None
		objectid = self.extractid(object)
		if mode=="map" and (objectid == newcontextid or objectid == callerid):
			if hasattr(self, 'NFGnav_ActiveMapItem'):
				return self.NFGnav_ActiveMapItem(client=PARENTS[0], REQUEST=REQUEST)
			else:
				return "<dt><img src=\"%s\" border=\"0\" hspace=\"5\"><b>%s</b></dt>" % (
					object.icon, object.title_or_id())
		elif mode=="map":
			if hasattr(self, 'NFGnav_LinkMapItem'):
				return self.NFGnav_LinkMapItem(client=PARENTS[0], REQUEST=REQUEST)
			else:
				retval = "<dt><img src=\"%s\" border=\"0\" hspace=\"5\">" % object.icon
				retval = retval + "<a href=\"%s\">%s</a></dt>" % (
					itemURL, object.title_or_id())
				return retval
		elif mode=="tree" and (objectid == newcontextid or objectid == callerid):
			if hasattr(self, 'NFGnav_ActiveTreeItem'):
				return self.NFGnav_ActiveTreeItem(client=PARENTS[0], REQUEST=REQUEST)
			else:
				return "<li><b><a href=\"%s\">%s</a></b></li>" % (
					itemURL, object.title_or_id())				
		elif mode=="tree":
			if hasattr(self, 'NFGnav_LinkTreeItem'):
				return self.NFGnav_LinkTreeItem(client=PARENTS[0], REQUEST=REQUEST)
			else:
				return "<li><a href=\"%s\">%s</a></li>" % (
					itemURL, object.title_or_id())
		elif mode=="trail" and (objectid == newcontextid or objectid == callerid):
			if hasattr(self, 'NFGnav_ActiveTrailItem'):
				return self.NFGnav_ActiveTrailItem(client=PARENTS[0], REQUEST=REQUEST)
			else:
				return "<a href=\"%s\">%s</a> : " % (itemURL, object.title_or_id())
		elif mode=="trail": # this clause should not be triggered, a LevelTrail has only Active objects
			if hasattr(self, 'NFGnav_LinkTrailItem'):  
				return self.NFGnav_LinkTrailItem(client=PARENTS[0], REQUEST=REQUEST)
			else:
				return "<a href=\"%s\">%s</a> : " % (itemURL, object.title_or_id())
		else:
			return "<!-- Unknown mode rendering request: %s -->" % mode

	def LevelOpen(self, PARENTS, REQUEST, level, mode="map"):
		"""
		Scriptable hook for level changes in a site map or tree menu.
		You can provide DTML methods for customized implementation:
		>> NFGnav_OpenMapLevel will be called when entering a SiteMap level.
		>> NFGnav_OpenTreeLevel will be called when entering a TreeMenu level.
		In both cases, the NFGnav_Level variable will be set to the 'new' level.
		If DTML methods are not found, a default rendering is returned.
		"""
		REQUEST.set('NFGnav_Level', level)
		if mode=="map" and  hasattr(self, 'NFGnav_OpenMapLevel'):
			return self.NFGnav_OpenMapLevel(client=PARENTS[0], REQUEST=REQUEST)
		elif mode=="map":
			return "<dl>"
		elif mode=="tree"and hasattr(self, 'NFGnav_OpenTreeLevel'):
			return self.NFGnav_OpenTreeLevel(client=PARENTS[0], REQUEST=REQUEST)
		elif mode=="trail" and hasattr(self, 'NFGnav_OpenTrailLevel'):
			return self.NFGnav_OpenTrailLevel(client=PARENTS[0], REQUEST=REQUEST)
		elif mode=="trail":
				return ""
		else:
			return "<ul>"

	def LevelClose(self, PARENTS, REQUEST, level, mode="map"):
		"""Scriptable hook for level changes in a site map or tree menu.
		You can provide DTML methods for customized implementation:
		>> NFGnav_CloseMapLevel will be called when closing a SiteMap level.
		>> NFGnav_CloseTreeLevel will be called when closing a TreeMenu level.
		In both cases, the NFGnav_Level variable will be set to the 'old' level.
		If DTML methods are not found, a default rendering is returned.
		"""
		REQUEST.set('NFGnav_Level', level)
		if mode=="map" and hasattr(self, 'NFGnav_CloseMapLevel'):
			return self.NFGnav_CloseMapLevel(client=PARENTS[0], REQUEST=REQUEST)
		elif mode=="map":
			return "</dl>"
		elif mode=="tree" and hasattr(self, 'NFGnav_CloseTreeLevel'):
			return self.NFGnav_CloseTreeLevel(client=PARENTS[0], REQUEST=REQUEST)
		elif mode=="trail" and hasattr(self, 'NFGnav_CloseTrailLevel'):
			return self.NFGnav_CloseTrailLevel(client=PARENTS[0], REQUEST=REQUEST)
		elif mode=="trail":
				return ""
		else:
			return "</ul>"

	def extractid(self, object):
		"""Folder id's are string type, while DTML Document id's are methods.
		This causes havoc in the code unless we explicitly check the type."""
		from types import *
		if type(object.id) == StringType:
			return object.id
		else:
			return object.id()
