# This code is copyrighted by bluedynamics.com and iuveno AG in June, 2001.
#
# written by Stephan Richter
# Last updated: June 8, 2001

from OFS.Image import File
from Globals import HTMLFile, MessageDialog
import string
import os

# The value separator that is used to separate the fields in the file 
seperator = '|||'


class ImportExportSupport:
    '''ImportExportSupport class - Support class to allow import and export of ZBabelTower data'''

    manage_options = ({'label': 'Export/Import', 'action': 'manage_importExport'},)

    manage_importExport = HTMLFile('dtml/manage_importExport', globals())


    def exportData(self):
        '''exportData(self) --> exports the data to a single text string; return string'''
        text = ''
        for phrase in self.getAllPhrases():
            # all we need to save are these three fields
            for field in ['md5hash', 'lang', 'phrase']:
                text = ((text + phrase[field]) + seperator)

            text = (text + '\n')

        return text


    def importData(self, text):
        '''importData(self, text) --> imports the data from a single text data string; return None'''
        rows = filter(None, string.split(text, (seperator + '\012')))

        for row in rows:
            fields = string.split(row, seperator)
            self.addPhrase(fields[0], fields[1], fields[2])



    def exportToFile(self, fileId = 'ZBabelDictionary_txt'):
        '''exportToFile(self, fileId=\'ZBabelDictionary_txt\') --> export the data to a ZODB File; return File'''
        text = self.exportData()
        return File(fileId, 'ZBabel Export File', text)


    def importFromFile(self, file):
        '''importFromFile(self, file) --> retrieve data from a ZODB File; return None'''
        self.importData(file.data)


    def exportToFolder(self, folder = None, fileId = 'ZBabelDictionary_txt', REQUEST = None):
        '''exportToFolder(self, folder=None, fileId=\'ZBabelDictionary_txt\', REQUEST=None) --> save the data
           in a file inside the specified directory; return HTML'''
        if (not folder):
            folder = self.aq_parent
        folder._setObject(fileId, self.exportToFile(fileId))
        if REQUEST:
            return MessageDialog(title=('Data exported to %s' % fileId),
                                 message=('All of the translations in this ZBabelTower contains has been exported to' +
                                          '%s' % os.path.join(folder.absolute_url(), fileId)), action='./manage_main')


    def importFromFolder(self, folder = None, fileId = 'ZBabelDictionary_txt', REQUEST = None):
        '''importFromFolder(self, folder=None, fileId=\'ZBabelDictionary_txt\', REQUEST=None) --> import the data
           from a specified file in a specific directory; return HTML'''
        if (not folder):
            folder = self.aq_parent

        self.importFromFile(folder[fileId])
        
        if REQUEST:
            return MessageDialog(title=('Data imported from %s' % fileId),
                                 message='All of the found translations have been imported!', action='./manage_main')

