# This code is copyrighted by bluedynamics.com and iuveno AG in June, 2001.
#
# written by Stephan Richter
# Last updated: June 8, 2001

from Globals import HTMLFile
import string

class LabelSupport:
    '''LabelSupport class - Support for a special language called \'label\' '''

    manage_options = ({'label': 'Translate', 'action': 'manage_labels'},)

    manage_labels = HTMLFile('dtml/manage_labels', globals())


    def getAllLabels(self, condition = '%'):
        '''getAllLabels(self) --> get all labels from the database; return list'''
        return self.getAllPhrasesOfLanguage('label', condition)


    def getEditLanguages(self, REQUEST):
        '''getEditLanguages(self, REQUEST) --> get the languages that are selected for editing'''
        if (hasattr(REQUEST, 'cookies') and REQUEST.cookies.has_key('langs')):
            return filter(None, string.split(REQUEST.cookies['langs'], ','))
        else:
            return []


    def manage_changeEditLanguages(self, REQUEST):
        '''manage_changeEditLanguages(self, REQUEST) --> change languages that are being displayed
           for editing; return HTML'''
        if (not REQUEST.form.has_key('langs')):
            REQUEST.form['langs'] = []

        # write the languages in the cookie, so it memorizes it
        REQUEST['RESPONSE'].setCookie('langs', string.join(REQUEST.form['langs'], ','))
        REQUEST.cookies['langs'] = string.join(REQUEST.form['langs'], ',')

        if REQUEST:
            return self.manage_labels(self, REQUEST)


    def manage_addLabelLanguage(self, lang = None, REQUEST = None):
        '''manage_addLabelLanguage(self, REQUEST) --> add a new language for all the labels;
          return HTML'''
        if (not lang):
            lang = string.strip(REQUEST['lang'])

        if lang:
            # add a phrase for each label for the new language
            for label in self.getAllLabels():
                self.addPhrase(label.md5hash, lang, label.phrase)

        if REQUEST:
            return self.manage_labels(self, REQUEST)


    def manage_deleteLabelLanguages(self, langs = None, REQUEST = None):
        '''manage_deleteLabelLanguages(self, REQUEST) --> delete the specified languages; return HTML'''
        if (not langs):
            langs = REQUEST.form['langs']

        editLangs = self.getEditLanguages(REQUEST)

        for lang in langs:
            # make sure all the phrases for this language is deleted
            self.deletePhrasesOfLanaguage(lang)
            if (lang in editLangs):
                editLangs.remove(lang)

        REQUEST.form['langs'] = editLangs
        return self.manage_changeEditLanguages(REQUEST)


    def manage_addLabels(self, REQUEST):
        '''manage_addLabels(self, REQUEST) --> add new labels to the database; return HTML'''
        hashDict = {}
        for key in REQUEST.form.keys():
            if ((key[:3] == 'new') and (key[4:9] == 'label')):
                hashDict[string.split(key, '-')[-1]] = self._makeDigest(REQUEST.form[key])

        for key in REQUEST.form.keys():
            if ((key[:3] == 'new') and REQUEST.form[key]):
                (garbage, lang, number,) = string.split(key, '-')
                if (number in hashDict.keys()):
                    self.addPhrase(hashDict[number], lang, REQUEST.form[key])

        if REQUEST:
            return self.manage_labels(self, REQUEST)


    def manage_editLabels(self, REQUEST):
        '''manage_editLabels(self, REQUEST) --> edit the labels; return HTML'''
        for key in REQUEST.form.keys():
            if (key[:2] == 'p-'):
                (garbage, lang, id,) = string.split(key, '-')
                phrase = self.getPhrase(id)
                if (phrase and (phrase.phrase == REQUEST.form[key])):
                    self.editPhrase(id, REQUEST.form[key])
                elif (not phrase):
                    label = self.getDstPhraseFromSrcPhrase(id, 'label')
                    self.newPhrase(label.md5hash, lang, REQUEST.form[key])

        self.manage_addLabels(REQUEST)
        if REQUEST:
            return self.manage_labels(self, REQUEST)


    def manage_deleteLabels(self, REQUEST):
        '''manage_deleteLabels(self, REQUEST) --> delete label and all translations; return HTML'''
        if REQUEST.form.has_key('labelIds'):
            for id in REQUEST.form['labelIds']:
                self.deletePhraseAndTranslations(id)

        if REQUEST:
            return self.manage_labels(self, REQUEST)

