# This code is copyrighted by bluedynamics.com and iuveno AG in June, 2001.
#
# written by Stephan Richter
# Last updated: June 8, 2001

from OFS.Folder import Folder
from OFS.ObjectManager import ObjectManager
from Globals    import HTMLFile
import string, Products

languageMap = {'en-us': 'en', 'de': 'de', 'en': 'en', 'fr': 'fr'}

icons = {'DTML Document': 'misc_/OFSP/dtmldoc.gif',
         'DTML Method'  : 'misc_/OFSP/dtmlmethod.gif',
         'File'         : 'misc_/OFSP/File_icon.gif',
         'Folder'       : 'misc_/OFSP/Folder_icon.gif',
         'Image'        : 'misc_/OFSP/Image_icon.gif'}

# get the folder's __getattr__; yeah, yeah, I stole this from Transparent Folders 
try: folder__getattr__ = ObjectManager.__getattr__
except:
    try:
        folder__getattr__ = ObjectManager.__getattr__
    except:
        try:
            folder__getattr__ = ObjectManager.inheritedAttribute('__getattr__')
        except:
            def folder__getattr__(self, name):
                raise AttributeError, name


manage_addZBabelObjectForm = HTMLFile('dtml/manage_addZBabelObjectForm', globals())

def manage_addZBabelObject(self, id, title=None, meta_type='Image', REQUEST=None):
    """manage_addZBabelObject(self, id, title=None, meta_type='Image', REQUEST=None) --> add a Babel Tower
       to the ZODB; return None or HTML"""

    ob = ZBabelObject(id, title, meta_type)
    self._setObject(id, ob)

    if REQUEST:
        ob = getattr(self, id)
        meta_type = ob.all_meta_types()[0]
        return REQUEST.RESPONSE.redirect(ob.absolute_url()+'/'+meta_type['action'])
    


class ZBabelObject(Folder):
    '''ZBabelObject class '''

    meta_type = 'ZBabel Object'

    _v_alreadyLooking = 0

    def __init__(self, id, title='', meta_type='Image'):
        '''__init__(self, id, title='', meta_type=\'Image\') --> initialize the object; return None'''
        self.id    = id
        self.title = title + ' *Localized*'
        self.icon  = icons[meta_type]
        self.contain_meta_type = meta_type
        

    def __call__(self):
        '''__call__(self) --> call the objects; redirect the call to the localized object; return obejct'''
        return self._getTrueObject()


    def __str__(self):
        '''__str__(self) --> return the string representation of the object; return None'''
        return str(self._getTrueObject())


    __repr__ = __call__

    
    def __getattr__(self, name):
        '''__getattr__(self, name) --> try to get an attribute from the localized object; return attr'''
        if not self._v_alreadyLooking and name[:3] != '_p_':
            try:
                self._v_alreadyLooking  = 1
                # NOTE: THIS CODE IS STILL NOT WORKING, SINCE THERE IS NO WAY TO HAVE REQUEST OR ACQUISITION
                #       WORKING HERE. IF YOU HAVE A CLUE, SEND ME A MAIL ***PLEASE***!
                #object = self._getTrueObject()
                #if hasattr(object, name):
                #    return getattr(object, name)
            finally:
                self._v_alreadyLooking = 0
        return folder__getattr__(self, name)


    def _checkId(self, id, allow_dup=0):
        '''_checkId(self, id, allow_dup=0) --> Check the id. There should be only objects in this folder
           which are representing a language'''
        if id not in languageMap.values():
            raise 'Bad Request', ('The id "%s" is invalid - not in language list.'  % id)  


    folder_all_meta_types = Folder.all_meta_types
    def all_meta_types(self):
        '''all_meta_types(self) --> return the meta type which represents this localized object; return list'''
        for meta_type in self.folder_all_meta_types():
            if meta_type['name'] == self.contain_meta_type:
                return [meta_type]

        return []


    def _getTrueObject(self):
        '''_getTrueObject(self) --> Get the object which is actually suppose to be shown; return object'''
        try: lang = self.findPreferredLanguage()
        except: lang = None
        if lang and lang in self.objectIds():
            object = self[lang]
        elif len(self.objectIds()):
            object = self.objectValues()[0]
        else:
            return None

        return object.__of__(self)


    def findPreferredLanguage(self, REQUEST={}):
        '''findPreferredLanguage(self, REQUEST) --> find the destination language from context; return string'''

        if not REQUEST:
            try: REQUEST = self.REQUEST
            except: pass

        try:
            lang = folder__getattr__(self, 'language')
        except AttributeError:
            if self.REQUEST.has_key('language'):
                lang = self.REQUEST['language']
            else:
                lang = string.split(self.REQUEST['HTTP_ACCEPT_LANGUAGE'], ',')[0]

        if lang in languageMap.keys():
            return languageMap[lang]
        else:
            return lang
