# This code is copyrighted by bluedynamics.com and iuveno AG in June, 2001.
#
# written by Stephan Richter
# Last updated: October 27, 2001

from OFS.Folder import Folder
from OFS.ObjectManager import ObjectManager
from Globals    import HTMLFile

try:
    from Globals import get_request 
except ImportError:
    print ( 'You have not installed the GlobalRequestPatch. ' +
            'ZBabel Objects will not be available' )
    
import string, Products
from ExtensionClass import Base

languageMap = {'en-us': 'en', 'de': 'de', 'en': 'en', 'fr': 'fr'}

icons = {'DTML Document': 'misc_/OFSP/dtmldoc.gif',
         'DTML Method'  : 'misc_/OFSP/dtmlmethod.gif',
         'File'         : 'misc_/OFSP/File_icon.gif',
         'Folder'       : 'misc_/OFSP/Folder_icon.gif',
         'Image'        : 'misc_/OFSP/Image_icon.gif'}


class Rewrapper (Base):
    def __init__(self, path):
        self._path = path

    def __of__(self, parent):
        ob = parent
        for p in self._path:
            ob = getattr(ob, p)
        return ob



manage_addZBabelObjectForm = HTMLFile('dtml/manage_addZBabelObjectForm', globals())

def manage_addZBabelObject(self, id, title=None, meta_type='Image', REQUEST=None):
    """manage_addZBabelObject(self, id, title=None, meta_type='Image', REQUEST=None) --> add a Babel Tower
       to the ZODB; return None or HTML"""

    ob = ZBabelObject(id, title, meta_type)
    self._setObject(id, ob)

    if REQUEST:
        ob = getattr(self, id)
        meta_type = ob.all_meta_types()[0]
        return REQUEST.RESPONSE.redirect(ob.absolute_url()+'/'+meta_type['action'])
    


class ZBabelObject(Folder):
    '''ZBabelObject class '''

    meta_type = 'ZBabel Object'

    _v_alreadyLooking = 0

    def __init__(self, id, title='', meta_type='Image'):
        '''__init__(self, id, title='', meta_type=\'Image\') --> initialize the object; return None'''
        self.id    = id
        self.title = title + ' *Localized*'
        self.icon  = icons[meta_type]
        self.contain_meta_type = meta_type
        

    def __repr__(self):
        '''__repr__(self) --> call the objects; redirect the call to the localized object; return obejct'''
        obj = self._getTrueObject()
        obj = obj.__of__(self)
        return obj       


    def __call__(self, *params, **keywords):
        '''__call__(self) --> call the objects; redirect the call to the localized object; return obejct'''
        try:
            obj = self._getTrueObject()
            obj = obj.__of__(self)
            return apply(obj, params, keywords)
        except:
            return obj       

    
    def __str__(self):
        '''__str__(self) --> return the string representation of the object; return None'''
        obj = self._getTrueObject()
        obj = obj.__of__(self)
        return obj.__str__()
        

    def __getattr__(self, name):
        '''__getattr__(self, name) --> try to get an attribute from the localized object; return attr'''
        if name[0] != '_' and not self._v_alreadyLooking:
            try:
                self._v_alreadyLooking = 1
                trueObject = self._getTrueObject()
                if hasattr(trueObject, name):
                    return Rewrapper((trueObject.getId(), name))

            finally:
                self._v_alreadyLooking = 0                

        return Base.__getattr__(self, name)


    def _checkId(self, id, allow_dup=0):
        '''_checkId(self, id, allow_dup=0) --> Check the id. There should be only objects in this folder
           which are representing a language'''
        if id not in languageMap.values():
            raise 'Bad Request', ('The id "%s" is invalid - not in language list.'  % id)  


    folder_all_meta_types = Folder.all_meta_types
    def all_meta_types(self):
        '''all_meta_types(self) --> return the meta type which represents this localized object; return list'''
        for meta_type in self.folder_all_meta_types():
            if meta_type['name'] == self.contain_meta_type:
                return [meta_type]

        return []


    def _getTrueObject(self):
        '''_getTrueObject(self) --> Get the object which is actually suppose to be shown; return object'''

        lang = self.findPreferredLanguage(get_request())
        ids = map(lambda x: x['id'], self._objects)
        
        if lang not in ids:
            if 'en' in ids:
                lang = 'en'
            elif len(ids):
                lang = ids[0]
            else:
                lang = None
        
        if lang:
            return self.__dict__.get(lang, None)
            

    def findPreferredLanguage(self, REQUEST={}):
        '''findPreferredLanguage(self, REQUEST) --> find the destination language from context; return string'''

        if not REQUEST:
            try: REQUEST = get_request()
            except: pass

        try:
            lang = getattr(self, 'language')
        except AttributeError:
            if REQUEST.has_key('language'):
                lang = REQUEST['language']
            else:
                lang = string.split(REQUEST['HTTP_ACCEPT_LANGUAGE'], ',')[0]

        if lang in languageMap.keys():
            return languageMap[lang]
        else:
            return lang
