# This code is copyrighted by bluedynamics.com and iuveno AG in June, 2001.
#
# written by Andrew Milton and Stephan Richter
# Last updated: June 8, 2001

from DocumentTemplate.DT_Util import Eval, expr_globals, ParseError, render_blocks
import string
import regex

# Defines the possible ZBabelTower meta types
BabelTowerMetaTypes = ['ZBabel Tower (SQL)', 'ZBabel Tower (ZODB)']

# Defines the languages that can be simplified
LanguageMap = {'en-us': 'en', 'de': 'de', 'en': 'en'}


class BasicBabelTag:
    '''BasicBabelTag class - This is a generic ZBable translation class. It defines some of the
       boring tasks of a class'''

    # name of the tag
    name = 'tag'
    # possible attributes the tag can have and their default values
    attrs = {'literal': 0, 'src': 'label', 'dst': None, 'towerId': None}


    def _prepareSmartTranslation(self):
        '''_prepareSmartTranslation(self) --> prepare translation; return string'''
        rawText = ''
        for block in self.section.blocks:
            if (type(block).__name__ == 'string'):
                rawText = (rawText + block)
            else:
                rawText = (rawText + ('<<%i>>' % self.section.blocks.index(block)))

        return string.strip(rawText)


    def _afterSmartTranslation(self, text, md):
        '''_afterSmartTranslation(self, text, md) --> after translation; return string'''
        for block in self.section.blocks:
            if (type(block).__name__ != 'string'):
                blockIndex = self.section.blocks.index(block)
                renderedBlock = render_blocks([self.section.blocks[blockIndex]], md)
                text = string.replace(text, ('<<%i>>' % blockIndex), str(renderedBlock))

        return text


    def validateArguments(self, args):
        '''validateArguments(self, args) --> try to evaluate the passed expression or try
           to get an object from the passed id; if all this fails, leave the string, it is
           probably cool!; return tuple of (name, value)'''
        # I stole this from dtml-let...
        # SR: Like he said: Always copy existing code to make you life easier (even though
        #     I changed some variables around
        for count in range(len(args)):
            (name, expr,) = args[count]
            if ((expr[:1] == '"') and ((expr[-1:] == '"') and (len(expr) > 1))):
                expr = expr[1:-1]
                try:
                    args[count] = (name, Eval(expr, expr_globals).eval)
                except SyntaxError, v:
                    (m, (huh, l, c, src,),) = v
                    raise ParseError, (('<strong>Expression (Python) Syntax error</strong>:' +
                                        '<pre>\012%s\012</pre>\012' % v[0]), 'babel')

        return args


    def defineAttributes(self, md):
        '''defineAttributes(self, md) --> add the parsed attributes to the class; return None'''
        params = {}
        try:
            for (name, expr,) in self.args:
                if (type(expr) is type('')):
                    try: params[name] = md[expr]
                    except: params[name] = expr
                else:
                    params[name] = expr(md)
        finally:
            pass

        for param in self.attrs.keys():
            if params.has_key(param):
                setattr(self, param, params[param])



    def findPreferredLanguage(self, md):
        'findPreferredLanguage(self, md) --> find the destination language from context'
        if self.dst:
            try: return md[self.dst]
            except: return self.dst
        else:
            if md.has_key('language'):
                lang = md['language']
            else:
                lang = string.split(md['HTTP_ACCEPT_LANGUAGE'], ',')[0]
            if (lang in LanguageMap.keys()):
                return LanguageMap[lang]
            else:
                return lang


    def translate(self, srcText, md, makeNewEntries = 1):
        '''translate(self, srcText, md) --> translate the text; return string'''
        if hasattr(md, 'this'):
            object = md.this
        else:
            object = md
        _Translator = findBabelTower(object, self.towerId)
        if (not _Translator):
            if (hasattr(self, 'section') and self.section):
                return self.section(object, md)
            else:
                return srcText
        translation = _Translator.translate(srcText, self.src, self.findPreferredLanguage(md), makeNewEntries)
        return translation


    def smartTranslate(self, md, makeNewEntries = 1):
        '''translateLiteral(self, md, makeNewEntries = 1) --> translate'''
        srcText = self._prepareSmartTranslation()
        text = self.translate(srcText, md, makeNewEntries)
        text = self._afterSmartTranslation(text, md)
        return text


    def render(self, md):
        '''render(self, md) --> Do the Translation; return string'''
        pass


    def __call__(self, md):
        '''__call__(self, md) --> render the tag; do the translation; return string'''
        return self.render(md)


def findBabelTower(root, towerId = None):
    '''findBabelTower(root, towerId=None) --> find the next BabelTower object; return object'''
    alternative = None
    while (root is not None):
        if hasattr(root, 'objectValues'):
            for object in root.objectValues():
                try:
                    # In case no Babel Tower with the specified towerId is found, then we can
                    # always fall back on the alternative Babel Tower (the first one found)
                    if ((not alternative) and (hasattr(object, 'meta_type') and
                                               (object.meta_type in BabelTowerMetaTypes))):
                        alternative = object

                    # Okay, lwt's check whether this is the right one.
                    if ((hasattr(o, 'meta_type') and (object.meta_type in BabelTowerMetaTypes))
                        and ((not towerId) or (object.getId() == towerId))):
                        return object
                except:
                    pass

        if hasattr(root, 'aq_parent'):
            root = root.aq_parent
        else:
            root = None

    return alternative


def parseTagParameters(paramText, result = None, tag = 'babel',
                       parmre = regex.compile('\\([\000- ]*\\([^\000- ="]+\\)=\\([^\000- ="]+\\)\\)'),
                       qparmre = regex.compile('\\([\000- ]*\\([^\000- ="]+\\)="\\([^"]*\\)"\\)'), **parms):
    result = (result or [])

    # Parse parameters of the form: name=value
    if (parmre.match(paramText) >= 0):
        name = parmre.group(2)
        value = parmre.group(3)
        length = len(parmre.group(1))

    # Parse parameters of the form: name="value"
    elif (qparmre.match(paramText) >= 0):
        name = qparmre.group(2)
        value = ('"%s"' % qparmre.group(3))
        length = len(qparmre.group(1))

    else:
        # There are no more parameters to parse
        if ((not paramText) or (not string.strip(paramText))):
            return result
        raise ParseError, (('invalid parameter: "%s"' % paramText), tag)

    # add the parameter/value pait to the results
    result.append((name, value))

    # remove the found parameter from the paramText
    paramText = string.strip(paramText[length:])

    if paramText:
        return apply(parseTagParameters, (paramText, result, tag), parms)
    else:
        return result
