# This code is copyrighted by bluedynamics.com and iuveno AG in June, 2001.
#
# written by Stephan Richter
# Last updated: October 27, 2001

from Globals import HTMLFile
from AccessControl.Role import RoleManager
from OFS.SimpleItem import SimpleItem
from LabelSupport import LabelSupport
from ImportExportSupport import ImportExportSupport

import md5, base64
import string, types


class ZBabelTowerInterface(SimpleItem, RoleManager, LabelSupport, ImportExportSupport):
    '''ZBabelTowerInterface class  - A class that implementd the dictionary of the ZBabel Translation System'''
    
    meta_type = 'ZBabel Tower Interface'

    manage_options = ((LabelSupport.manage_options + ImportExportSupport.manage_options) +
                      ({'label': 'Security', 'action': 'manage_access'},
                       {'label': 'Undo', 'action': 'manage_UndoForm'}))

    __ac_permissions__ = (('Manage ZBabel Towers', ('manage_main', 'manage', 'manage_workspace',
                                                    'manage_Translate', 'reCatalog')),)

    manage_main = LabelSupport.manage_labels


    def __init__(self, id, title = None):
        '''__init__(self, id, connection, title=None) --> Inialize the object; return None'''
        self.id = id
        self.title = title


    def __getitem__(self, item):
        '''__getitem__(self, item) --> try to get the item in a dictionary-like syntax; return object'''
        if (type(item) is types.StringType):
            if hasattr(self, item):
                return getattr(self, item)
            else:
                return self.getPhrase(item)
        else:
            raise TypeError, 'The index must be a string.'


    # *** Start defining public ZBabel Tower API **

    def addPhrase(self, md5hash, lang, phrase, urls = []):
        '''addPhrase(self, md5hash, lang, phrase, urls=[]) --> add a phrase to the tower; return None'''
        pass


    def editPhrase(self, id, phrase, urls = []):
        '''editPhrase(self, id, phrase, urls=[]) --> edit a phrase in the tower; return None'''
        pass


    def deletePhrase(self, id):
        '''deletePhrase(self, id) --> delete a phrase from the tower; return None'''
        pass


    def deletePhraseAndTranslations(self, id):
        '''deletePhraseAndTranslations(self, id) --> delete a phrase and all its translations; return None'''
        pass


    def deletePhrasesOfLanaguage(self, lang):
        '''deletePhrasesOfLanaguage(self, lang) --> delete all phrases for language; return None'''
        pass


    def getPhrase(self, id):
        '''getPhrase(self, id) --> get phrase by id; return Phrase'''
        pass


    def getDstPhraseFromSrcPhrase(self, srcPhrase, dstLang):
        '''getDstPhraseFromSrcPhrase(self, srcPhrase, dstLang) --> Get the phrase which is the translation of
        the srcPhrase in language dstLang; return Phrase'''
        pass


    def getAllPhrases(self):
        '''getAllPhrases(self) --> return all phrases; return List'''
        pass


    def getAllPhrasesOfLanguage(self, lang, condition):
        '''getAllPhrasesOfLanguage(self, lang, condition) --> get all phrases of a particular language;
        return List'''
        pass


    def getAllLanguages(self):
        '''getAllLanguages(self) --> get a list of all languages; return list'''
        pass

    # *** End defining public ZBabel Tower API **


    def translate(self, srcPhrase, srcLang = 'label', dstLang = 'en', makeNewEntries = 1):
        '''translate(self, srcPhrase, srcLang="label", dstLang="en", makeNewEntries = 1) --> translate the
           srcText to language dstLang; return string'''
        srcPhrase = _cleanText(srcPhrase)
        digest = self._makeDigest(srcPhrase)
        matches = self.getPhrasesByHash(digest)
        if matches:
            for match in matches:
                if (match.lang == dstLang):
                    return match.phrase

        elif makeNewEntries:
            for language in self.getAllLanguages() + ['label']:
                self.addPhrase(digest, language, srcPhrase)

        return srcPhrase


    def _makeDigest(self, srcPhrase):
        '''_makeDigest(self, srcPhrase) -->  make a MD5 Hash Digest from the source text;
           return string'''
        digest = md5.new()
        digest.update(srcPhrase)
        digest = digest.digest()
        digest = string.strip(base64.encodestring(digest))
        return digest



def _cleanText(srcText):
    '''_cleanText(srcText) --> clean text from HTML dirt; return text'''
    textList = string.split(srcText, '\n')
    textList = map(string.strip, textList)
    srcText = string.join(textList, ' ')
    return srcText

