# This code is copyrighted by bluedynamics.com and iuveno AG in June, 2001.
#
# written by Stephan Richter
# Last updated: June 8, 2001

from Globals import HTMLFile
from ZBabelTowerInterface import ZBabelTowerInterface
from Phrase import Phrase

import string, regex


manage_addZODBBabelTowerForm = HTMLFile('dtml/manage_addZODBBabelTowerForm', globals())

def manage_addZODBBabelTower(self, id, title = None, REQUEST = None):
    '''manage_addZODBBabelTower(self, id, title=None, REQUEST=None) --> add a Babel Tower
     to the ZODB; return None or HTML'''
    ob = ZODBBabelTower(id, title)
    self._setObject(id, ob)
    if REQUEST:
        return self.manage_main(self, REQUEST)


class ZODBBabelTower(ZBabelTowerInterface,):
    '''ZODBBabelTower - A class that implementd the dictionary of the ZBabel Translation System '''

    # meta type of this ZBabel Tower
    meta_type = 'ZBabel Tower (ZODB)'


    def __init__(self, id, title = None):
        '''__init__(self, id, connection, title=None) --> Inialize the object; return None'''
        self.hashes = {}
        self.id = id
        self.title = title


    def _makeIdForPhrase(self, phrase):
        '''_makeId(self, lang) --> make a new '''
        if len(self.hashes):
            max = 0
            ids = self.getAllPhraseIds()
            for id in ids:
                count = int(string.split(id, '_')[1])
                if (count > max):
                    max = count

            max = (max + 1)
            return ((phrase.lang + '_') + str(max))
        else:
            return (phrase.lang + '_0')


    # *** Start implementing ZBabel Tower API ***

    def addPhrase(self, md5hash, lang, phrase, urls = []):
        '''addPhrase(self, md5hash, lang, phrase, urls=[]) --> add a phrase to the tower; return None'''
        phrase = Phrase(md5hash=md5hash, lang=lang, phrase=phrase)
        phrase.id = self._makeIdForPhrase(phrase)
        # make the phrase part of this object
        phrase._p_jar = self._p_jar
        phrase.__of__(self)
        setattr(self, phrase.id, phrase)

        if self.hashes.has_key(md5hash):
            self.hashes[md5hash].append(phrase.id)
        else:
            self.hashes[md5hash] = [phrase.id]


    def editPhrase(self, id, phrase = '', urls = []):
        '''editPhrase(self, id, phrase, urls=[]) --> edit a phrase in the tower; return None'''
        getattr(self, id).edit(phrase)


    def deletePhrase(self, id):
        '''deletePhrase(self, id) --> delete a phrase from the tower; return None'''
        phrase = getattr(self, id)
        self.hashes[phrase.md5hash].remove(id)
        if (not len(self.hashes[phrase.md5hash])):
            del self.hashes[phrase.md5hash]
        delattr(self, id)


    def deletePhraseAndTranslations(self, id):
        '''deletePhraseAndTranslations(self, id) --> delete a phrase and all its translations; return None'''
        md5hash = self.getPhrase(id).md5hash
        for phrase in self.getAllPhrases():
            if (phrase.md5hash == md5hash):
                self.deletePhrase(phrase.id)


    def deletePhrasesOfLanaguage(self, lang):
        '''deletePhrasesOfLanaguage(self, lang) --> delete all phrases for language; return None'''
        for id in self.getAllPhraseIds():
            if (string.split(id, '_')[1] == lang):
                self.deletePhrase(id)


    def getPhrase(self, id):
        '''getPhrase(self, id) --> get phrase by id; return Phrase'''
        return getattr(self, id)


    def getDstPhraseFromSrcPhrase(self, srcPhrase, dstLang):
        '''getDstPhraseFromSrcPhrase(self, srcPhraseId, dstLang) --> Get the phrase which is the translation of
           the srcPhrase in language dstLang; return Phrase'''
        srcHash = self._makeDigest(srcPhrase)
        for phrase in self.getPhrasesByHash(srcHash):
            if (phrase.lang == dstLang):
                return phrase



    def getAllPhrases(self):
        '''getAllPhrases(self) --> return all phrases; return List'''
        phrases = []
        for id in self.getAllPhraseIds():
            phrases.append(getattr(self, id))

        return phrases


    def getAllPhraseIds(self):
        '''getAllPhraseIds(self) --> return the ids of all phrases; return list'''
        return reduce(lambda x, y: (x + y), self.hashes.values(), [])


    def getAllPhrasesOfLanguage(self, lang, condition = '%'):
        '''getAllPhrasesOfLanguage(self, lang) --> get all phrases of a particular language; return List'''
        ids = self.getAllPhraseIds()
        condition = string.replace(condition, '%', '.*')
        cond = regex.compile(condition)
        phrases = []
        for id in ids:
            if (string.split(id, '_')[0] == lang):
                phrase = getattr(self, id)
                if (cond.match(phrase.phrase) >= 0):
                    phrases.append(phrase)

        return phrases


    def getAllLanguages(self):
        '''getAllLanguages(self) --> get a list of all languages; return list'''
        ids = self.getAllPhraseIds()
        langs = []
        for id in ids:
            lang = string.split(id, '_')[0]
            if ((lang not in langs) and (lang != 'label')):
                langs.append(lang)

        return langs


    def getPhrasesByHash(self, hash):
        '''getPhrasesByHash(self, hash) --> get all phrases with the same hash; return List'''
        try:
            return map(self.getPhrase, self.hashes[hash])
        except KeyError:
            return []

    # *** End implementing ZBabel Tower API ***











