# This code is copyrighted by bluedynamics.com and iuveno AG in June, 2001.
#
# written by Stephan Richter
# Last updated: June 8, 2001

from Globals import HTMLFile
from Products.ZSQLMethods.SQL import SQL
from types import StringType
from SQLContainerSupport import SQLContainerSupport
from ZBabelTowerInterface import ZBabelTowerInterface
from Phrase import Phrase

import ZBabelSQL
import string


manage_addZSQLBabelTowerForm = HTMLFile('dtml/manage_addZSQLBabelTowerForm', globals())

def manage_addZSQLBabelTower(self, id, connection, title = None, REQUEST = None):
    '''manage_addZBabelTower(self, id, connection, title = None, REQUEST=None) --> add a Babel Tower
       to the ZODB; return None or HTML'''
    ob = ZSQLBabelTower(id, connection, title)
    self._setObject(id, ob)

    if REQUEST:
        return self.manage_main(self, REQUEST)


class ZSQLBabelTower(ZBabelTowerInterface, SQLContainerSupport):
    '''ZSQLBabelTower - A class that implementd the dictionary of the ZBabel Translation System'''

    meta_type = 'ZBabel Tower (SQL)'

    __ac_permissions__ = (ZBabelTowerInterface.__ac_permissions__ + SQLContainerSupport.__ac_permissions__)

    manage_options = (ZBabelTowerInterface.manage_options + SQLContainerSupport.manage_options)

    sqlMethods = (('sql_initPhrases', 'Initialize the Phrases Table', 'dropFirst:int:required'),
                  ('sql_addPhrase', 'Add a New Phrase', 'md5hash:required lang:required phrase:required'),
                  ('sql_editPhrase', 'Edit a Single Phrase', 'id:int:required phrase:required'),
                  ('sql_deletePhrase', 'Delete a Single Phrase', 'id:int:required'),
                  ('sql_deletePhraseAndTranslations', 'Delete Phrase and its Translations', 'id:int:required'),
                  ('sql_deletePhrasesOfLanguage', 'Delete Phrases of Language', 'lang:string:required'),
                  ('qry_getAllPhrases', 'Get All Phrases', ''),
                  ('qry_getPhrase', 'Get Phrases For on Page', 'id:int:required'),
                  ('qry_getAllPhrasesOfLanguage', 'Get Existing Languages', 'lang:string:required condition:string="%"'),
                  ('qry_getDstPhraseFromSrcPhrase', 'Get list of Unique md5Hashes', ('srcPhrase:string:required ' +
                                                                                     'dstLang:string:required')),
                  ('qry_getPhrasesByHash', 'Find Matching Phrases', 'hash:string:required'),
                  ('qry_getAllLanguages', 'Get All Languages From DB', ''))


    def __init__(self, id, connection, title = None):
        '''__init__(self, id, connection, title=None) --> Inialize the object; return None'''
        self.connection = connection
        self.id = id
        self.title = title
        self._registerSQLQueries()


    def _registerSQLQueries(self):
        '''_registerSQLQueries(self) --> iniitialize all the SQL Methods; return None'''
        for method in self.sqlMethods:
            sql = getattr(ZBabelSQL, ('_' + method[0]))
            object = SQL(method[0], method[1], self.connection, method[2], sql)
            setattr(self, method[0], object)

        # Set the cache on this SQL Method, since it is used for the translation
        self.qry_getPhrasesByHash.manage_advanced(max_rows=5000, max_cache=5000,
                                                  cache_time=3600, class_name='', class_file='')


    def _addURLs(self, md5hash, urls):
        '''_addURLs(self, md5hash, urls) --> add URLs to the DB, if necessary; return None'''
        if urls:
            savedURLs = map(lambda row: row['url'], self.getURLsOfMD5Hash(md5hash))
            for url in urls:
                if (url not in savedURLs):
                    self.sql_addURL(md5hash, url)



    def _makePhraseFromDBRow(self, dbRow = None):
        '''_makePhraseFromDBRow(self, dbRow) --> from the DB row create the Phrase; return Phrase'''
        if dbRow:
            phrase = Phrase(dbRow=dbRow)
            phrase._p_jar = self._p_jar
            return phrase.__of__(self)
        else:
            return None


    def _makePhrasesFromDBRows(self, dbRows = []):
        '''_makePhrasesFromDBRows(self, dbRows=[]) --> from the DB rows create the Phrases; return list'''
        phrases = []
        for row in dbRows:
            phrase = self._makePhraseFromDBRow(row)
            if phrase:
                phrases.append(phrase)

        return phrases


    # *** Start implementing ZBabel Tower API ***

    def addPhrase(self, md5hash, lang, phrase, urls = []):
        '''addPhrase(self, md5hash, lang, phrase, urls=[]) --> add a phrase to the tower; return None'''
        id = self.sql_addPhrase(md5hash=md5hash, lang=lang, phrase=phrase)[0]['id']


    def editPhrase(self, id, phrase, urls = []):
        '''editPhrase(self, id, phrase, urls=[]) --> edit a phrase in the tower; return None'''
        self.sql_editPhrase(id=id, phrase=phrase)


    def deletePhrase(self, id):
        '''deletePhrase(self, id) --> delete a phrase from the tower; return None'''
        self.sql_deletePhrase(id=id)


    def deletePhraseAndTranslations(self, id):
        '''deletePhraseAndTranslations(self, id) --> delete a phrase and all its translations; return None'''
        self.sql_deletePhraseAndTranslations(id=id)


    def deletePhrasesOfLanaguage(self, lang):
        '''deletePhrasesOfLanaguage(self, lang) --> delete all phrases for language; return None'''
        self.sql_deletePhrasesOfLanguage(lang=lang)


    def getPhrase(self, id):
        '''getPhrase(self, id) --> get phrase by id; return Phrase'''
        dbRows = self.qry_getPhrase(id=id)
        try:
            return self._makePhraseFromDBRow(dbRows[0])
        except (IndexError, ValueError):
            return None


    def getDstPhraseFromSrcPhrase(self, srcPhrase, dstLang):
        '''getDstPhraseFromSrcPhrase(self, srcPhraseId, dstLang) --> Get the phrase which is the translation of
          the srcPhrase in language dstLang; return Phrase'''
        dbRows = self.qry_getDstPhraseFromSrcPhrase(srcPhrase=srcPhrase, dstLang=dstLang)
        try:
            return self._makePhraseFromDBRow(dbRows[0])
        except IndexError:
            return None


    def getAllPhrases(self):
        '''getAllPhrases(self) --> return all phrases; return List'''
        dbRows = self.qry_getAllPhrases()
        return self._makePhrasesFromDBRows(dbRows)


    def getAllPhrasesOfLanguage(self, lang, condition='%'):
        '''getAllPhrasesOfLanguage(self, lang, condition="%") --> get all phrases of a particular language;
           return List'''
        dbRows = self.qry_getAllPhrasesOfLanguage(lang=lang, condition=condition)
        return self._makePhrasesFromDBRows(dbRows)


    def getPhrasesByHash(self, hash):
        '''getPhrasesByHash(self, hash) --> get all phrases with the same hash; return List'''
        dbRows = self.qry_getPhrasesByHash(hash=hash)
        return self._makePhrasesFromDBRows(dbRows)


    def getAllLanguages(self):
        '''getAllLanguages(self) --> get a list of all languages; return list'''
        dbRows = self.qry_getAllLanguages()
        langs = map(lambda row: row['lang'], dbRows)
        if ('label' in langs):
            del langs[langs.index('label')]
        return langs

    # *** End implementing ZBabel Tower API ***
